/*
    *	This interface implements the easy communication through I2C-DEV 
    *
    *   Copyright (c)2019 Tóthpál István, istvan@tothpal.eu
*/

// TODO: Make this object as smart as it can control I2CLine and make the I2CLineBus object for the pure communication only

#include "i2cdevbus.h"
#include "i2clinebus.h"
#include "../businter.h"

#include "../e2cmdw.h"
#include "../eeptypes.h"


I2CDevBus::I2CDevBus(BusInterface *p) {
    err_no=0;
    last_addr=0;
//	shot_delay=5;
	busI=p;
};

I2CDevBus::~I2CDevBus() {
};

int I2CDevBus::Open(int port) {

//    printf("I2C DEV Bus Open port: %d",port);
    int err = busI->Open(port);
    if (err == 0) {
        epchiptype = cmdWin->GetCurrentChipType();
        epbanksize = GetEEPTypeSize(epchiptype);

        switch ( epchiptype ) {
            case E2402:
                printf("E2402 chip number of banks: %d\n", GetEEPTypeSize(epchiptype) );
                is_16bit_i2c_eprom = 0;
                break;
            case E2408:
                printf("E2408 chip number of banks: %d\n", GetEEPTypeSize(epchiptype) );
                is_16bit_i2c_eprom = 0;
                break;
            case E24256:
                printf("E24256 chip number of banks: %d\n", GetEEPTypeSize(epchiptype) );
                is_16bit_i2c_eprom = 1;
                break;
            default:
                printf("0x%06X chip number of banks: %d\n", epchiptype, epbanksize );         
        };
        static_cast<I2CLineBus*>(busI)->Set16bit(is_16bit_i2c_eprom);
//        printf("succeeded! \n");
    }
/*    else {
        printf("NOT succeeded! \n");
    };
*/    return err;
};

void I2CDevBus::Close() {
    busI->Close();
//    printf("I2C DEV Bus Close port\n");
};

int I2CDevBus::TestPort(int port) {
    busI->TestPort(port);
    return 0;
};

int I2CDevBus::Reset() {
    return 0;
};

int I2CDevBus::Stop() {
    return 0;
};

//TODO: We may try to read block first and if it not succeeded then read bytes
//TODO: Take control of linebus, linebus just need to communicate
long I2CDevBus::Read(int addr, uint8_t *data, long length, int page_size) {

    I2CLineBus* b = static_cast<I2CLineBus*>(busI);

    if (is_16bit_i2c_eprom==0) { // 8-bit eeprom
        if ( (addr >= 160) & (addr <(160 + 2*epbanksize)) ) {
            if ( b->Read( int(addr/2), data, length, page_size ) == 0) {
//                printf("I2C DEV Bus READ DATA: %d %d %d\n", int(addr/2), length, page_size);
                return length;  // read byte count!
            }
        };
    }
    else { // 16-bit eeprom
        if ( addr == 160 ) {
            if ( b->Read( int(addr/2), data, length, page_size ) == 0) {
//                printf("I2C DEV Bus READ DATA: %d %d %d\n", int(addr/2), length, page_size);
                return length;  // read byte count!
            }
        };
    };

//    printf("I2C DEV Bus READ DATA - error: %d %d %d\n", addr, length, page_size);
    return 0;
};


//TODO: Take control of linebus, linebus just need to communicate
long I2CDevBus::Write(int addr, uint8_t const *data, long length, int page_size) {

    I2CLineBus* b = static_cast<I2CLineBus*>(busI);
    
    if ( (addr >= 160) & (addr <176) ) { 
        switch (length) {
            case 2:
                if ( b->WriteWord( int(addr/2), data, length, page_size ) == 0) {
                    SetLastProgrammedAddress( data[0] );
//                    printf("I2C DEV Bus WRITE DATA: %d %d - data: %d position:%d\n", int(addr/2), length, data[1], data[0]);
                    return length;  // read byte count!
                }
                break;

            case 1: return 0;
                    break;

            default:
                    if ( b->WriteBlock( int(addr/2), data, length, page_size ) == 0) {
                        SetLastProgrammedAddress( data[0]*256+data[1]+length-3 );
//                        printf("I2C DEV Bus WRITE DATA: %d %d - data: %d position:%d\n", int(addr/2), length, data[1], data[0]);
                        return length;  // read byte count!
                    }
                    break;
        }
    };

    printf("I2C DEV Bus WRITE: %d %d %d\n", addr & 0xff, data[0], data[1]);
    return 0;
};

int I2CDevBus::Start(uint8_t slave) {
    printf("I2C DEV Bus Start slave: %d \n", slave);
    return 0;
};

int I2CDevBus::ReadByte(int ack, int lsb) {
    printf("I2C DEV Bus READByte: %d %d\n", ack, lsb);
    return 0;
};

int I2CDevBus::WriteByte(int by, int lsb) {
    printf("I2C DEV Bus WRITEByte: %d %d \n",by, lsb);
    return 0;
};

long I2CDevBus::StartRead(uint8_t slave, uint8_t *data, long length) {
    printf("I2C DEV Bus StartRead: %d %d\n", slave, length);
    return 0;
};

long I2CDevBus::StartWrite(uint8_t slave, uint8_t const *data, long length) {
    I2CLineBus* b = static_cast<I2CLineBus*>(busI);
    
    if ((slave == 160) & (length==2)) { //& (slave < 176) 
        if ( b->StartWrite( int(slave/2), data, length ) == length) {
//            SetLastProgrammedAddress( data[0] );
            return length;  // read byte count!
        }
    };
    if ((slave >= 160) & (slave < 176) & (length==1)) {
            return length;  // read byte count!
    }
    printf("I2C DEV Bus StartWrite: %d %d %d\n", slave, length, data[0]);
    return 0;
};

void I2CDevBus::SetDelay() {
};

