/*
    *	This interface implements the easy communication through I2C-DEV 
    *
    *   Copyright (c)2019 Tóthpál István, istvan@tothpal.eu
*/

#ifndef _I2C_LINE_BUS_
#define _I2C_LINE_BUS_

#include "../businter.h"
#include <linux/i2c.h>

struct memaddr_16bit_struct {
    int addrH;
    int addrL;
};


class I2CLineBus : public BusInterface
{
    public:
	I2CLineBus();
	virtual ~I2CLineBus();
	
        int Open(int port);
        void Close();

        int TestOpen(int port);
        int TestPort(int port);
        int TestSave(int port);
        void TestRestore();
        int SetPower(bool onoff);
        void SetControlLine(int res = 1);
        void SetDataOut(int sda = 1);
        void SetInvDataOut(int sda = 1);
        void SetClock(int scl = 1);
        int GetDataIn();
        int GetClock();
        void SetClockData();
        void ClearClockData();
        int IsClockDataUP();
        int IsClockDataDOWN();

    void Set16bit(int is16bit);
        
	int Read(int addr, uint8_t *data, long length, int page_size);
    int i2c_read_8bit(__u8 slave_addr, __u8 memaddr, __u8 *result);         // with  8bit device addressing
    int i2c_read_16bit(__u8 slave_addr, __u16 memaddr, __u8 *result);       // with 16bit device addressing
    
    int WriteWord(int addr, const uint8_t *data, long length, int page_size);       // data[0] - position, data[1] - data
    int WriteBlock(int addr, const uint8_t *data, long length, int page_size);      // data[0] - position, data[1] - data
    int i2c_write_block_16bit(__u8 slave_addr, const __u8 *data, long length);      // with 16bit device addressing
    long StartWrite(uint8_t slave, const uint8_t *data, long length);               // changeing bank??

    private:

	int myI2CLine;		 // the opened line //openfile//

	int i2cbusnum;		 // my CH341 can be accessed at /dev/i2c-4 on my Mageia 6 machine and /dev/i2c-8 at my Debian 7 laptop
	int i2cdevaddr;		 // device address on i2c line, 24x02 mostly at 0x50
	
	int is_16bit_i2c_eprom = 0;
    struct memaddr_16bit_struct memaddr_16bit;
	int clockdata;
    int dataout;
};

#endif
